package gyak8;

import java.util.*;

public class Alkalmazott {
	private String nev;
	private String azonosito;
	private int alapFizetes;
	private ArrayList<Alkalmazott> beoszottak;
	private Alkalmazott fonoke;
	
	private static final Comparator<Alkalmazott> NEVCOMP = new Comparator<Alkalmazott>() {
		public int compare(Alkalmazott elso, Alkalmazott masodik) {
			return elso.getNev().compareTo(masodik.getNev()); 
		}
	};
	
	public String getNev() {
		return this.nev;
	}
	
	
	public Alkalmazott(String nev, String azonosito, int alapFizetes) { 
		this.nev = nev;
		this.azonosito = azonosito;
		this.alapFizetes = alapFizetes;
		this.beoszottak = new ArrayList<Alkalmazott>();
		this.fonoke = null;
		
	}
	
	
	
	public int getFizetes() {
		return (this.alapFizetes * osszesBeosztottSzama()) + 5000;
	}
	

	public String toString() {
		return "nev:" + this.nev + ",  fizetes:" + getFizetes();
	}

	
	@Override
	public int hashCode() {
		final int PRIME = 31;
		int result = 1;
		result = PRIME * result + ((azonosito == null) ? 0 : azonosito.hashCode());
		return result;
	}

	
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		
		if (obj == null)
			return false;
		
		if (getClass() != obj.getClass())
			return false;
	
		final Alkalmazott other = (Alkalmazott) obj;
		if (azonosito == null) {
			if (other.azonosito != null)
				return false;
		} else if (!azonosito.equals(other.azonosito))
			return false;
		return true;
	}
	
	
	public boolean hazzaadBeoszott(Alkalmazott beoszt) {
		if(beoszt == null)
			return false;
		
		if(this.beoszottak.contains(beoszt))  /* ha mar benne van */
			return false;
		
		this.beoszottak.add(beoszt);
		
		if(beoszt.fonoke != null)				/* ha mar van fonoke */
			beoszt.fonoke.beoszottak.remove(beoszt);
		
		beoszt.fonoke = this;
		return true;	
	}
	
	
	public void beoszottTorol(Alkalmazott torlendo) {
		if(torlendo == null)
			return;
	
		boolean volt = this.beoszottak.remove(torlendo);
		if(volt)
			torlendo.fonoke = null;
	}
	
	
	public void setFonok(Alkalmazott fonok) {
		if(this.fonoke == fonok)
			return;
		
		if(fonok != null)
			fonok.beoszottak.add(this);
	
		if(this.fonoke != null)					/* ha mar volt fonok */
			this.fonoke.beoszottak.remove(this);
		
		this.fonoke = fonok;
	}
	
	
	public Alkalmazott getFonok() {
		return this.fonoke;
	}
	
	
	public int kozvetlenBeosztottakSzama() {
		return this.beoszottak.size();
	}
	
	
	public Alkalmazott[] kozvetlenBeosztottakABCben() {
		Alkalmazott[] tomb = new Alkalmazott[this.beoszottak.size()];
		this.beoszottak.toArray(tomb);
		Arrays.sort(tomb, NEVCOMP);
		return tomb;
	}
	
	
	public int osszesBeosztottSzama() {
		int db = kozvetlenBeosztottakSzama();
		
		for(Alkalmazott alk : this.beoszottak)
			db+= alk.osszesBeosztottSzama();
		return db;
	}
}